#!/usr/local/bin/perl
# $Revision: 1.3 $$Date: 2001/04/07 00:24:42 $

# This tries to interpret a genbank file and heuristically
# turn it into something that pipmaker can use.

use 5.6.0;
use strict;
use warnings;
use Boulder::Genbank;

# Get Boulder from
#  http://stein.cshl.org/software/boulder/
#  http://bioperl.org/ScriptCentral/

my $argv0 = $0;
$argv0 =~ s{.*/}{};
my $filename = "";

sub strand
{
	return "Left" if $_[0] =~ /complement/;
	return "Right";
}

sub posns
{
	my $p = shift;
	my @x = ($p);

	if ( $p =~ /complement\((.+)\)/ ) {
		$p = $1;
		@x = ($p);
	}
	if ( $p =~ /join\((.+)\)/ ) {
		@x = split(/,/, $1);
	}
	map {s/[^\d]/ /g} @x;
	map {s/^ //g} @x;
	return @x;
}

sub leftpos
{
	my @p = posns($_[0]);
	$p[0] =~ /(\d+)\s+(\d+)/;
	return $1;
}

sub rightpos
{
	my @p = posns($_[0]);
	$p[$#p] =~ /(\d+)\s+(\d+)/;
	return $2;
}

my @map = (
	["Alu",     "Alu"],
	["MIR",     "MIR"],
	["L2",      "LINE2"],
	["L1",      "LINE1"],
	["LTR",     "LTR"],
	["LTR",     "LTR"],
	["HERV",    "LTR"],
	["DNA",     "DNA"],
	["B1",      "B1"],
	["B2",      "B2"],
	["SINE",    "SINE"],
	["LINE",    "Other"],
	["MML",     "Other"],
	["BUR1",    "Other"],
	["Other",   "Other"],
	["Unknown", "Other"],
	["RNA",     "RNA"]
);

sub normalize
{
	$_[0] = "" unless defined $_[0];
	for my $p (@map) {
		my $e = $p->[0] . ".*";
		return $p->[1] if $_[0] =~ /$e/;
	}
	printf STDERR "Warning: unknown element \"$_[0]\"\n";
	return "Unknown";
}

sub sayLRpos
{
	my ($A,$p,$C) = @_;
	print leftpos($p), " ", rightpos($p), " $A ", normalize($C), "\n";
}

# --- main ---

my $narg = @ARGV;
if ($narg == 1) {
	$filename = $ARGV[0];
} else {
	print STDERR "Usage:  $argv0 genbank_file > output_file\n";
	exit 1;
}
if ($filename eq '-') { $filename = '/dev/stdin'; }

my $gb = new Boulder::Genbank(-accessor=>'File',-param=>$filename) or die;

print "%:repeats\n";
while (my $s = $gb->get) {
	next unless $s && $s->Features;
	my @rpts = $s->Features->Repeat_region;

	foreach (@rpts) {
		my $p = $_->Position;
		my $g = $_->Rpt_family;
		sayLRpos strand($p), $p, $g;
		# print join("\n",posns($p)), "\n\n";
	}
}

